@@ -2,20 +2,19 @@ import $ from 'jquery';
 import angular from 'angular';
 import _ from 'lodash';
 
-var $win = $(window);
+const $win = $(window);
 
-$.fn.place_tt = (function() {
-  var defaults = {
+$.fn.place_tt = (() => {
+  const defaults = {
     offset: 5,
   };
 
-  return function(x, y, opts) {
+  return function(this: any, x: number, y: number, opts: any) {
     opts = $.extend(true, {}, defaults, opts);
 
-    return this.each(function() {
-      var $tooltip = $(this),
-        width,
-        height;
+    return this.each(() => {
+      const $tooltip = $(this);
+      let width, height;
 
       $tooltip.addClass('grafana-tooltip');
 
@@ -29,8 +28,8 @@ $.fn.place_tt = (function() {
           .invoke([
             '$compile',
             '$rootScope',
-            function($compile, $rootScope) {
-              var tmpScope = $rootScope.$new(true);
+            ($compile, $rootScope) => {
+              const tmpScope = $rootScope.$new(true);
               _.extend(tmpScope, opts.scopeData);
 
               $compile($tooltip)(tmpScope);
@@ -43,8 +42,11 @@ $.fn.place_tt = (function() {
       width = $tooltip.outerWidth(true);
       height = $tooltip.outerHeight(true);
 
-      $tooltip.css('left', x + opts.offset + width > $win.width() ? x - opts.offset - width : x + opts.offset);
-      $tooltip.css('top', y + opts.offset + height > $win.height() ? y - opts.offset - height : y + opts.offset);
+      const left = x + opts.offset + width > $win.width() ? x - opts.offset - width : x + opts.offset;
+      const top = y + opts.offset + height > $win.height() ? y - opts.offset - height : y + opts.offset;
+
+      $tooltip.css('left', left > 0 ? left : 0);
+      $tooltip.css('top', top > 0 ? top : 0);
     });
   };
 })();
