@@ -1,28 +1,38 @@
-import angular from 'angular';
+import angular, { ILocationService } from 'angular';
 import _ from 'lodash';
 
 import config from 'app/core/config';
 import coreModule from 'app/core/core_module';
-import { importPluginModule } from './plugin_loader';
 
-import { UnknownPanelCtrl } from 'app/plugins/panel/unknown/module';
+import { DataSourceApi } from '@grafana/data';
+import { importPanelPlugin, importDataSourcePlugin, importAppPlugin } from './plugin_loader';
+import DatasourceSrv from './datasource_srv';
+import { GrafanaRootScope } from 'app/routes/GrafanaCtrl';
 
-/** @ngInject **/
-function pluginDirectiveLoader($compile, datasourceSrv, $rootScope, $q, $http, $templateCache) {
-  function getTemplate(component) {
+/** @ngInject */
+function pluginDirectiveLoader(
+  $compile: any,
+  datasourceSrv: DatasourceSrv,
+  $rootScope: GrafanaRootScope,
+  $http: any,
+  $templateCache: any,
+  $timeout: any,
+  $location: ILocationService
+) {
+  function getTemplate(component: { template: any; templateUrl: any }) {
     if (component.template) {
-      return $q.when(component.template);
+      return Promise.resolve(component.template);
     }
-    var cached = $templateCache.get(component.templateUrl);
+    const cached = $templateCache.get(component.templateUrl);
     if (cached) {
-      return $q.when(cached);
+      return Promise.resolve(cached);
     }
-    return $http.get(component.templateUrl).then(res => {
+    return $http.get(component.templateUrl).then((res: any) => {
       return res.data;
     });
   }
 
-  function relativeTemplateUrlToAbs(templateUrl, baseUrl) {
+  function relativeTemplateUrlToAbs(templateUrl: string, baseUrl: string) {
     if (!templateUrl) {
       return undefined;
     }
@@ -32,11 +42,11 @@ function pluginDirectiveLoader($compile, datasourceSrv, $rootScope, $q, $http, $
     return baseUrl + '/' + templateUrl;
   }
 
-  function getPluginComponentDirective(options) {
+  function getPluginComponentDirective(options: any) {
     // handle relative template urls for plugin templates
     options.Component.templateUrl = relativeTemplateUrlToAbs(options.Component.templateUrl, options.baseUrl);
 
-    return function() {
+    return () => {
       return {
         templateUrl: options.Component.templateUrl,
         template: options.Component.template,
@@ -45,7 +55,7 @@ function pluginDirectiveLoader($compile, datasourceSrv, $rootScope, $q, $http, $
         controllerAs: 'ctrl',
         bindToController: true,
         scope: options.bindings,
-        link: (scope, elem, attrs, ctrl) => {
+        link: (scope: any, elem: any, attrs: any, ctrl: any) => {
           if (ctrl.link) {
             ctrl.link(scope, elem, attrs, ctrl);
           }
@@ -57,8 +67,8 @@ function pluginDirectiveLoader($compile, datasourceSrv, $rootScope, $q, $http, $
     };
   }
 
-  function loadPanelComponentInfo(scope, attrs) {
-    var componentInfo: any = {
+  function loadPanelComponentInfo(scope: any, attrs: any) {
+    const componentInfo: any = {
       name: 'panel-plugin-' + scope.panel.type,
       bindings: { dashboard: '=', panel: '=', row: '=' },
       attrs: {
@@ -68,15 +78,9 @@ function pluginDirectiveLoader($compile, datasourceSrv, $rootScope, $q, $http, $
       },
     };
 
-    let panelInfo = config.panels[scope.panel.type];
-    var panelCtrlPromise = Promise.resolve(UnknownPanelCtrl);
-    if (panelInfo) {
-      panelCtrlPromise = importPluginModule(panelInfo.module).then(function(panelModule) {
-        return panelModule.PanelCtrl;
-      });
-    }
-
-    return panelCtrlPromise.then(function(PanelCtrl: any) {
+    const panelInfo = config.panels[scope.panel.type];
+    return importPanelPlugin(panelInfo.id).then(panelPlugin => {
+      const PanelCtrl = panelPlugin.angularPanelCtrl;
       componentInfo.Component = PanelCtrl;
 
       if (!PanelCtrl || PanelCtrl.registered) {
@@ -84,7 +88,7 @@ function pluginDirectiveLoader($compile, datasourceSrv, $rootScope, $q, $http, $
       }
 
       if (PanelCtrl.templatePromise) {
-        return PanelCtrl.templatePromise.then(res => {
+        return PanelCtrl.templatePromise.then((res: any) => {
           return componentInfo;
         });
       }
@@ -93,7 +97,7 @@ function pluginDirectiveLoader($compile, datasourceSrv, $rootScope, $q, $http, $
         PanelCtrl.templateUrl = relativeTemplateUrlToAbs(PanelCtrl.templateUrl, panelInfo.baseUrl);
       }
 
-      PanelCtrl.templatePromise = getTemplate(PanelCtrl).then(template => {
+      PanelCtrl.templatePromise = getTemplate(PanelCtrl).then((template: any) => {
         PanelCtrl.templateUrl = null;
         PanelCtrl.template = `<grafana-panel ctrl="ctrl" class="panel-height-helper">${template}</grafana-panel>`;
         return componentInfo;
@@ -103,84 +107,94 @@ function pluginDirectiveLoader($compile, datasourceSrv, $rootScope, $q, $http, $
     });
   }
 
-  function getModule(scope, attrs) {
+  function getModule(scope: any, attrs: any): any {
     switch (attrs.type) {
       // QueryCtrl
       case 'query-ctrl': {
-        let datasource = scope.target.datasource || scope.ctrl.panel.datasource;
-        return datasourceSrv.get(datasource).then(ds => {
-          scope.datasource = ds;
+        const ds: DataSourceApi = scope.ctrl.datasource as DataSourceApi;
 
-          return importPluginModule(ds.meta.module).then(dsModule => {
-            return {
-              baseUrl: ds.meta.baseUrl,
-              name: 'query-ctrl-' + ds.meta.id,
-              bindings: { target: '=', panelCtrl: '=', datasource: '=' },
-              attrs: {
-                target: 'target',
-                'panel-ctrl': 'ctrl.panelCtrl',
-                datasource: 'datasource',
-              },
-              Component: dsModule.QueryCtrl,
-            };
-          });
+        return Promise.resolve({
+          baseUrl: ds.meta.baseUrl,
+          name: 'query-ctrl-' + ds.meta.id,
+          bindings: { target: '=', panelCtrl: '=', datasource: '=' },
+          attrs: {
+            target: 'ctrl.target',
+            'panel-ctrl': 'ctrl',
+            datasource: 'ctrl.datasource',
+          },
+          Component: ds.components.QueryCtrl,
         });
       }
       // Annotations
       case 'annotations-query-ctrl': {
-        return importPluginModule(scope.ctrl.currentDatasource.meta.module).then(function(dsModule) {
+        const baseUrl = scope.ctrl.currentDatasource.meta.baseUrl;
+        const pluginId = scope.ctrl.currentDatasource.meta.id;
+
+        return importDataSourcePlugin(scope.ctrl.currentDatasource.meta).then(dsPlugin => {
           return {
-            baseUrl: scope.ctrl.currentDatasource.meta.baseUrl,
-            name: 'annotations-query-ctrl-' + scope.ctrl.currentDatasource.meta.id,
+            baseUrl,
+            name: 'annotations-query-ctrl-' + pluginId,
             bindings: { annotation: '=', datasource: '=' },
             attrs: {
               annotation: 'ctrl.currentAnnotation',
               datasource: 'ctrl.currentDatasource',
             },
-            Component: dsModule.AnnotationsQueryCtrl,
+            Component: dsPlugin.components.AnnotationsQueryCtrl,
           };
         });
       }
       // Datasource ConfigCtrl
       case 'datasource-config-ctrl': {
-        var dsMeta = scope.ctrl.datasourceMeta;
-        return importPluginModule(dsMeta.module).then(function(dsModule): any {
-          if (!dsModule.ConfigCtrl) {
-            return { notFound: true };
-          }
+        const dsMeta = scope.ctrl.datasourceMeta;
+        const angularUrl = $location.url();
+        return importDataSourcePlugin(dsMeta).then(dsPlugin => {
+          scope.$watch(
+            'ctrl.current',
+            () => {
+              // This watcher can trigger when we navigate away due to late digests
+              // This check is to stop onModelChanged from being called when navigating away
+              // as it triggers a redux action which comes before the angular $routeChangeSucces and
+              // This makes the bridgeSrv think location changed from redux before detecting it was actually
+              // changed from angular.
+              if (angularUrl === $location.url()) {
+                scope.onModelChanged(scope.ctrl.current);
+              }
+            },
+            true
+          );
 
           return {
             baseUrl: dsMeta.baseUrl,
             name: 'ds-config-' + dsMeta.id,
             bindings: { meta: '=', current: '=' },
             attrs: { meta: 'ctrl.datasourceMeta', current: 'ctrl.current' },
-            Component: dsModule.ConfigCtrl,
+            Component: dsPlugin.angularConfigCtrl,
           };
         });
       }
       // AppConfigCtrl
       case 'app-config-ctrl': {
-        let model = scope.ctrl.model;
-        return importPluginModule(model.module).then(function(appModule) {
+        const model = scope.ctrl.model;
+        return importAppPlugin(model).then(appPlugin => {
           return {
             baseUrl: model.baseUrl,
             name: 'app-config-' + model.id,
             bindings: { appModel: '=', appEditCtrl: '=' },
             attrs: { 'app-model': 'ctrl.model', 'app-edit-ctrl': 'ctrl' },
-            Component: appModule.ConfigCtrl,
+            Component: appPlugin.angularConfigCtrl,
           };
         });
       }
       // App Page
       case 'app-page': {
-        let appModel = scope.ctrl.appModel;
-        return importPluginModule(appModel.module).then(function(appModule) {
+        const appModel = scope.ctrl.appModel;
+        return importAppPlugin(appModel).then(appPlugin => {
           return {
             baseUrl: appModel.baseUrl,
             name: 'app-page-' + appModel.id + '-' + scope.ctrl.page.slug,
             bindings: { appModel: '=' },
             attrs: { 'app-model': 'ctrl.appModel' },
-            Component: appModule[scope.ctrl.page.component],
+            Component: appPlugin.angularPages[scope.ctrl.page.component],
           };
         });
       }
@@ -189,15 +203,15 @@ function pluginDirectiveLoader($compile, datasourceSrv, $rootScope, $q, $http, $
         return loadPanelComponentInfo(scope, attrs);
       }
       default: {
-        return $q.reject({
+        return Promise.reject({
           message: 'Could not find component type: ' + attrs.type,
         });
       }
     }
   }
 
-  function appendAndCompile(scope, elem, componentInfo) {
-    var child = angular.element(document.createElement(componentInfo.name));
+  function appendAndCompile(scope: any, elem: JQuery, componentInfo: any) {
+    const child = angular.element(document.createElement(componentInfo.name));
     _.each(componentInfo.attrs, (value, key) => {
       child.attr(key, value);
     });
@@ -206,16 +220,19 @@ function pluginDirectiveLoader($compile, datasourceSrv, $rootScope, $q, $http, $
     elem.empty();
 
     // let a binding digest cycle complete before adding to dom
-    setTimeout(function() {
-      elem.append(child);
-      scope.$applyAsync(function() {
-        scope.$broadcast('component-did-mount');
-        scope.$broadcast('refresh');
+    setTimeout(() => {
+      scope.$applyAsync(() => {
+        elem.append(child);
+        setTimeout(() => {
+          scope.$applyAsync(() => {
+            scope.$broadcast('component-did-mount');
+          });
+        });
       });
     });
   }
 
-  function registerPluginComponent(scope, elem, attrs, componentInfo) {
+  function registerPluginComponent(scope: any, elem: JQuery, attrs: any, componentInfo: any) {
     if (componentInfo.notFound) {
       elem.empty();
       return;
@@ -228,8 +245,8 @@ function pluginDirectiveLoader($compile, datasourceSrv, $rootScope, $q, $http, $
     }
 
     if (!componentInfo.Component.registered) {
-      var directiveName = attrs.$normalize(componentInfo.name);
-      var directiveFn = getPluginComponentDirective(componentInfo);
+      const directiveName = attrs.$normalize(componentInfo.name);
+      const directiveFn = getPluginComponentDirective(componentInfo);
       coreModule.directive(directiveName, directiveFn);
       componentInfo.Component.registered = true;
     }
@@ -239,13 +256,12 @@ function pluginDirectiveLoader($compile, datasourceSrv, $rootScope, $q, $http, $
 
   return {
     restrict: 'E',
-    link: function(scope, elem, attrs) {
+    link: (scope: any, elem: JQuery, attrs: any) => {
       getModule(scope, attrs)
-        .then(function(componentInfo) {
+        .then((componentInfo: any) => {
           registerPluginComponent(scope, elem, attrs, componentInfo);
         })
-        .catch(err => {
-          $rootScope.appEvent('alert-error', ['Plugin Error', err.message || err]);
+        .catch((err: any) => {
           console.log('Plugin component error', err);
         });
     },
