@@ -1,14 +1,17 @@
 import _ from 'lodash';
+import flatten from 'app/core/utils/flatten';
 import * as queryDef from './query_def';
 import TableModel from 'app/core/table_model';
+import { DataQueryResponse, DataFrame, toDataFrame, FieldType, MutableDataFrame } from '@grafana/data';
+import { ElasticsearchAggregation } from './types';
 
 export class ElasticResponse {
-  constructor(private targets, private response) {
+  constructor(private targets: any, private response: any) {
     this.targets = targets;
     this.response = response;
   }
 
-  processMetrics(esAgg, target, seriesList, props) {
+  processMetrics(esAgg: any, target: any, seriesList: any, props: any) {
     let metric, y, i, newSeries, bucket, value;
 
     for (y = 0; y < target.metrics.length; y++) {
@@ -88,6 +91,7 @@ export class ElasticResponse {
             datapoints: [],
             metric: metric.type,
             field: metric.field,
+            metricId: metric.id,
             props: props,
           };
           for (i = 0; i < esAgg.buckets.length; i++) {
@@ -109,7 +113,7 @@ export class ElasticResponse {
     }
   }
 
-  processAggregationDocs(esAgg, aggDef, target, table, props) {
+  processAggregationDocs(esAgg: any, aggDef: ElasticsearchAggregation, target: any, table: any, props: any) {
     // add columns
     if (table.columns.length === 0) {
       for (const propKey of _.keys(props)) {
@@ -119,7 +123,7 @@ export class ElasticResponse {
     }
 
     // helper func to add values to value array
-    const addMetricValue = (values, metricName, value) => {
+    const addMetricValue = (values: any[], metricName: string, value: any) => {
       table.addColumn({ text: metricName });
       values.push(value);
     };
@@ -155,6 +159,14 @@ export class ElasticResponse {
             }
             break;
           }
+          case 'percentiles': {
+            const percentiles = bucket[metric.id].values;
+
+            for (const percentileName in percentiles) {
+              addMetricValue(values, `p${percentileName} ${metric.field}`, percentiles[percentileName]);
+            }
+            break;
+          }
           default: {
             let metricName = this.getMetricName(metric.type);
             const otherMetrics = _.filter(target.metrics, { type: metric.type });
@@ -175,9 +187,9 @@ export class ElasticResponse {
   }
 
   // This is quite complex
-  // need to recurise down the nested buckets to build series
-  processBuckets(aggs, target, seriesList, table, props, depth) {
-    let bucket, aggDef, esAgg, aggId;
+  // need to recurse down the nested buckets to build series
+  processBuckets(aggs: any, target: any, seriesList: any, table: any, props: any, depth: any) {
+    let bucket, aggDef: any, esAgg, aggId;
     const maxDepth = target.bucketAggs.length - 1;
 
     for (aggId in aggs) {
@@ -212,8 +224,8 @@ export class ElasticResponse {
     }
   }
 
-  private getMetricName(metric) {
-    let metricDef = _.find(queryDef.metricAggTypes, { value: metric });
+  private getMetricName(metric: any) {
+    let metricDef: any = _.find(queryDef.metricAggTypes, { value: metric });
     if (!metricDef) {
       metricDef = _.find(queryDef.extendedStats, { value: metric });
     }
@@ -221,13 +233,13 @@ export class ElasticResponse {
     return metricDef ? metricDef.text : metric;
   }
 
-  private getSeriesName(series, target, metricTypeCount) {
+  private getSeriesName(series: any, target: any, metricTypeCount: any) {
     let metricName = this.getMetricName(series.metric);
 
     if (target.alias) {
       const regex = /\{\{([\s\S]+?)\}\}/g;
 
-      return target.alias.replace(regex, (match, g1, g2) => {
+      return target.alias.replace(regex, (match: any, g1: any, g2: any) => {
         const group = g1 || g2;
 
         if (group.indexOf('term ') === 0) {
@@ -240,7 +252,7 @@ export class ElasticResponse {
           return metricName;
         }
         if (group === 'field') {
-          return series.field;
+          return series.field || '';
         }
 
         return match;
@@ -248,11 +260,27 @@ export class ElasticResponse {
     }
 
     if (series.field && queryDef.isPipelineAgg(series.metric)) {
-      const appliedAgg = _.find(target.metrics, { id: series.field });
-      if (appliedAgg) {
-        metricName += ' ' + queryDef.describeMetric(appliedAgg);
+      if (series.metric && queryDef.isPipelineAggWithMultipleBucketPaths(series.metric)) {
+        const agg: any = _.find(target.metrics, { id: series.metricId });
+        if (agg && agg.settings.script) {
+          metricName = agg.settings.script;
+
+          for (const pv of agg.pipelineVariables) {
+            const appliedAgg: any = _.find(target.metrics, { id: pv.pipelineAgg });
+            if (appliedAgg) {
+              metricName = metricName.replace('params.' + pv.name, queryDef.describeMetric(appliedAgg));
+            }
+          }
+        } else {
+          metricName = 'Unset';
+        }
       } else {
-        metricName = 'Unset';
+        const appliedAgg: any = _.find(target.metrics, { id: series.field });
+        if (appliedAgg) {
+          metricName += ' ' + queryDef.describeMetric(appliedAgg);
+        } else {
+          metricName = 'Unset';
+        }
       }
     } else if (series.field) {
       metricName += ' ' + series.field;
@@ -275,7 +303,7 @@ export class ElasticResponse {
     return name.trim() + ' ' + metricName;
   }
 
-  nameSeries(seriesList, target) {
+  nameSeries(seriesList: any, target: any) {
     const metricTypeCount = _.uniq(_.map(seriesList, 'metric')).length;
 
     for (let i = 0; i < seriesList.length; i++) {
@@ -284,15 +312,17 @@ export class ElasticResponse {
     }
   }
 
-  processHits(hits, seriesList) {
-    const series = {
+  processHits(hits: { total: { value: any }; hits: any[] }, seriesList: any[]) {
+    const hitsTotal = typeof hits.total === 'number' ? hits.total : hits.total.value; // <- Works with Elasticsearch 7.0+
+
+    const series: any = {
       target: 'docs',
       type: 'docs',
       datapoints: [],
-      total: hits.total,
+      total: hitsTotal,
       filterable: true,
     };
-    let propName, hit, doc, i;
+    let propName, hit, doc: any, i;
 
     for (i = 0; i < hits.hits.length; i++) {
       hit = hits.hits[i];
@@ -317,8 +347,8 @@ export class ElasticResponse {
     seriesList.push(series);
   }
 
-  trimDatapoints(aggregations, target) {
-    const histogram = _.find(target.bucketAggs, { type: 'date_histogram' });
+  trimDatapoints(aggregations: any, target: any) {
+    const histogram: any = _.find(target.bucketAggs, { type: 'date_histogram' });
 
     const shouldDropFirstAndLast = histogram && histogram.settings && histogram.settings.trimEdges;
     if (shouldDropFirstAndLast) {
@@ -332,7 +362,7 @@ export class ElasticResponse {
     }
   }
 
-  getErrorFromElasticResponse(response, err) {
+  getErrorFromElasticResponse(response: any, err: any) {
     const result: any = {};
     result.data = JSON.stringify(err, null, 4);
     if (err.root_cause && err.root_cause.length > 0 && err.root_cause[0].reason) {
@@ -364,7 +394,7 @@ export class ElasticResponse {
       if (response.aggregations) {
         const aggregations = response.aggregations;
         const target = this.targets[i];
-        const tmpSeriesList = [];
+        const tmpSeriesList: any[] = [];
         const table = new TableModel();
 
         this.processBuckets(aggregations, target, tmpSeriesList, table, {}, 0);
@@ -383,4 +413,158 @@ export class ElasticResponse {
 
     return { data: seriesList };
   }
+
+  getLogs(logMessageField?: string, logLevelField?: string): DataQueryResponse {
+    const dataFrame: DataFrame[] = [];
+
+    for (let n = 0; n < this.response.responses.length; n++) {
+      const response = this.response.responses[n];
+      if (response.error) {
+        throw this.getErrorFromElasticResponse(this.response, response.error);
+      }
+
+      const { propNames, docs } = flattenHits(response.hits.hits);
+      if (docs.length > 0) {
+        const series = createEmptyDataFrame(propNames, this.targets[0].timeField, logMessageField, logLevelField);
+
+        // Add a row for each document
+        for (const doc of docs) {
+          if (logLevelField) {
+            // Remap level field based on the datasource config. This field is then used in explore to figure out the
+            // log level. We may rewrite some actual data in the level field if they are different.
+            doc['level'] = doc[logLevelField];
+          }
+
+          series.add(doc);
+        }
+
+        dataFrame.push(series);
+      }
+
+      if (response.aggregations) {
+        const aggregations = response.aggregations;
+        const target = this.targets[n];
+        const tmpSeriesList: any[] = [];
+        const table = new TableModel();
+
+        this.processBuckets(aggregations, target, tmpSeriesList, table, {}, 0);
+        this.trimDatapoints(tmpSeriesList, target);
+        this.nameSeries(tmpSeriesList, target);
+
+        for (let y = 0; y < tmpSeriesList.length; y++) {
+          const series = toDataFrame(tmpSeriesList[y]);
+          dataFrame.push(series);
+        }
+      }
+    }
+
+    return { data: dataFrame };
+  }
 }
+
+type Doc = {
+  _id: string;
+  _type: string;
+  _index: string;
+  _source?: any;
+};
+
+/**
+ * Flatten the docs from response mainly the _source part which can be nested. This flattens it so that it is one level
+ * deep and the keys are: `level1Name.level2Name...`. Also returns list of all properties from all the docs (not all
+ * docs have to have the same keys).
+ * @param hits
+ */
+const flattenHits = (hits: Doc[]): { docs: Array<Record<string, any>>; propNames: string[] } => {
+  const docs: any[] = [];
+  // We keep a list of all props so that we can create all the fields in the dataFrame, this can lead
+  // to wide sparse dataframes in case the scheme is different per document.
+  let propNames: string[] = [];
+
+  for (const hit of hits) {
+    const flattened = hit._source ? flatten(hit._source, null) : {};
+    const doc = {
+      _id: hit._id,
+      _type: hit._type,
+      _index: hit._index,
+      _source: { ...flattened },
+      ...flattened,
+    };
+
+    for (const propName of Object.keys(doc)) {
+      if (propNames.indexOf(propName) === -1) {
+        propNames.push(propName);
+      }
+    }
+
+    docs.push(doc);
+  }
+
+  propNames.sort();
+  return { docs, propNames };
+};
+
+/**
+ * Create empty dataframe but with created fields. Fields are based from propNames (should be from the response) and
+ * also from configuration specified fields for message, time, and level.
+ * @param propNames
+ * @param timeField
+ * @param logMessageField
+ * @param logLevelField
+ */
+const createEmptyDataFrame = (
+  propNames: string[],
+  timeField: string,
+  logMessageField?: string,
+  logLevelField?: string
+): MutableDataFrame => {
+  const series = new MutableDataFrame({ fields: [] });
+
+  series.addField({
+    name: timeField,
+    type: FieldType.time,
+  });
+
+  if (logMessageField) {
+    series.addField({
+      name: logMessageField,
+      type: FieldType.string,
+    }).parse = (v: any) => {
+      return v || '';
+    };
+  } else {
+    series.addField({
+      name: '_source',
+      type: FieldType.string,
+    }).parse = (v: any) => {
+      return JSON.stringify(v, null, 2);
+    };
+  }
+
+  if (logLevelField) {
+    series.addField({
+      name: 'level',
+      type: FieldType.string,
+    }).parse = (v: any) => {
+      return v || '';
+    };
+  }
+
+  const fieldNames = series.fields.map(field => field.name);
+
+  for (const propName of propNames) {
+    // Do not duplicate fields. This can mean that we will shadow some fields.
+    if (fieldNames.includes(propName)) {
+      continue;
+    }
+
+    series.addField({
+      name: propName,
+      type: FieldType.string,
+    }).parse = (v: any) => {
+      return v || '';
+    };
+  }
+
+  return series;
+};
